@@ -29,6 +29,7 @@
 
 #include "common/spi-dbus.h"
 #include "object.h"
+#include "introspection.h"
 
 static dbus_bool_t
 impl_get_CharacterCount (DBusMessageIter * iter, void *user_data)
@@ -369,7 +370,7 @@ impl_GetAttributes (DBusConnection * bus, DBusMessage * message,
   if (reply)
     {
       dbus_message_iter_init_append (reply, &iter);
-      spi_atk_append_attribute_set (&iter, set);
+      spi_object_append_attribute_set (&iter, set);
       dbus_message_append_args (reply, DBUS_TYPE_INT32, &startOffset,
                                 DBUS_TYPE_INT32, &endOffset,
                                 DBUS_TYPE_INVALID);
@@ -397,7 +398,7 @@ impl_GetDefaultAttributes (DBusConnection * bus, DBusMessage * message,
   if (reply)
     {
       dbus_message_iter_init_append (reply, &iter);
-      spi_atk_append_attribute_set (&iter, set);
+      spi_object_append_attribute_set (&iter, set);
     }
   atk_attribute_set_free (set);
   return reply;
@@ -793,7 +794,7 @@ impl_GetDefaultAttributeSet (DBusConnection * bus, DBusMessage * message,
   if (reply)
     {
       dbus_message_iter_init_append (reply, &iter);
-      spi_atk_append_attribute_set (&iter, attributes);
+      spi_object_append_attribute_set (&iter, attributes);
     }
 
   if (attributes)
@@ -836,5 +837,5 @@ void
 spi_initialize_text (DRoutePath * path)
 {
   droute_path_add_interface (path,
-                             SPI_DBUS_INTERFACE_TEXT, methods, properties);
+                             SPI_DBUS_INTERFACE_TEXT, spi_org_a11y_atspi_Text, methods, properties);
 };
